@@ -8,6 +8,7 @@ import os
 import pickle
 import shutil
 from test import check_present, without_import
+from unittest.mock import MagicMock, patch
 
 import numpy as np
 import pandas as pd
@@ -76,14 +77,58 @@ def test_from_csv_files(caplog: pytest.LogCaptureFixture) -> None:
     )
 
     # draws_pd()
-    assert bern_gqs.draws_pd().shape == (400, 10)
+    assert bern_gqs.draws_pd().shape == (400, 13)
     assert (
         bern_gqs.draws_pd(inc_sample=True).shape[1]
         == bern_gqs.previous_fit.draws_pd().shape[1]
         + bern_gqs.draws_pd().shape[1]
+        - 3  # chain, iter, draw duplicates
     )
 
-    assert list(bern_gqs.draws_pd(vars=['y_rep']).columns) == column_names
+    assert list(bern_gqs.draws_pd(vars=['y_rep']).columns) == (column_names)
+
+    assert list(
+        bern_gqs.draws_pd(vars=["chain__", "iter__", "draw__", 'y_rep']).columns
+    ) == (["chain__", "iter__", "draw__"] + column_names)
+
+
+def test_pd_xr_agreement():
+    # fitted_params sample - list of filenames
+    goodfiles_path = os.path.join(DATAFILES_PATH, 'runset-good', 'bern')
+    csv_files = []
+    for i in range(4):
+        csv_files.append('{}-{}.csv'.format(goodfiles_path, i + 1))
+
+    # gq_model
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli_ppc.stan')
+    model = CmdStanModel(stan_file=stan)
+    jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+
+    bern_gqs = model.generate_quantities(data=jdata, previous_fit=csv_files)
+
+    draws_pd = bern_gqs.draws_pd(inc_sample=True)
+    draws_xr = bern_gqs.draws_xr(inc_sample=True)
+
+    # check that the indexing is the same between the two
+    np.testing.assert_equal(
+        draws_pd[draws_pd['chain__'] == 2]['y_rep[1]'],
+        draws_xr.y_rep.sel(chain=2).isel(y_rep_dim_0=0).values,
+    )
+    # "draw" is 0-indexed in xarray, equiv. "iter__" is 1-indexed in pandas
+    np.testing.assert_equal(
+        draws_pd[draws_pd['iter__'] == 100]['y_rep[1]'],
+        draws_xr.y_rep.sel(draw=99).isel(y_rep_dim_0=0).values,
+    )
+
+    # check for included sample as well
+    np.testing.assert_equal(
+        draws_pd[draws_pd['chain__'] == 2]['theta'],
+        draws_xr.theta.sel(chain=2).values,
+    )
+    np.testing.assert_equal(
+        draws_pd[draws_pd['iter__'] == 100]['theta'],
+        draws_xr.theta.sel(draw=99).values,
+    )
 
 
 def test_from_csv_files_bad() -> None:
@@ -152,16 +197,17 @@ def test_from_previous_fit_draws() -> None:
 
     bern_gqs = model.generate_quantities(data=jdata, previous_fit=bern_fit)
 
-    assert bern_gqs.draws_pd().shape == (400, 10)
+    assert bern_gqs.draws_pd().shape == (400, 13)
     assert (
         bern_gqs.draws_pd(inc_sample=True).shape[1]
         == bern_gqs.previous_fit.draws_pd().shape[1]
         + bern_gqs.draws_pd().shape[1]
+        - 3  # duplicates of chain, iter, and draw
     )
     row1_sample_pd = bern_fit.draws_pd().iloc[0]
     row1_gqs_pd = bern_gqs.draws_pd().iloc[0]
     np.testing.assert_array_equal(
-        pd.concat((row1_sample_pd, row1_gqs_pd), axis=0).values,
+        pd.concat((row1_sample_pd, row1_gqs_pd), axis=0).values[3:],
         bern_gqs.draws_pd(inc_sample=True).iloc[0].values,
     )
     # draws_xr
@@ -210,9 +256,9 @@ def test_from_previous_fit_variables() -> None:
         bern_gqs.stan_variable(var='lp__')
 
     vars_dict = bern_gqs.stan_variables()
-    var_names = list(
-        bern_gqs.previous_fit.metadata.stan_vars_cols.keys()
-    ) + list(bern_gqs.metadata.stan_vars_cols.keys())
+    var_names = list(bern_gqs.previous_fit.metadata.stan_vars.keys()) + list(
+        bern_gqs.metadata.stan_vars.keys()
+    )
     assert set(var_names) == set(list(vars_dict.keys()))
 
 
@@ -266,9 +312,9 @@ def test_save_warmup(caplog: pytest.LogCaptureFixture) -> None:
         10,
     )
 
-    assert bern_gqs.draws_pd().shape == (400, 10)
-    assert bern_gqs.draws_pd(inc_warmup=False).shape == (400, 10)
-    assert bern_gqs.draws_pd(inc_warmup=True).shape == (800, 10)
+    assert bern_gqs.draws_pd().shape == (400, 13)
+    assert bern_gqs.draws_pd(inc_warmup=False).shape == (400, 13)
+    assert bern_gqs.draws_pd(inc_warmup=True).shape == (800, 13)
     assert bern_gqs.draws_pd(vars=['y_rep'], inc_warmup=False).shape == (
         400,
         10,
@@ -285,9 +331,9 @@ def test_save_warmup(caplog: pytest.LogCaptureFixture) -> None:
         bern_gqs.stan_variable(var='lp__')
 
     vars_dict = bern_gqs.stan_variables()
-    var_names = list(
-        bern_gqs.previous_fit.metadata.stan_vars_cols.keys()
-    ) + list(bern_gqs.metadata.stan_vars_cols.keys())
+    var_names = list(bern_gqs.previous_fit.metadata.stan_vars.keys()) + list(
+        bern_gqs.metadata.stan_vars.keys()
+    )
     assert set(var_names) == set(list(vars_dict.keys()))
 
     xr_data = bern_gqs.draws_xr()
@@ -522,7 +568,7 @@ def test_from_optimization() -> None:
     assert bern_gqs.draws(inc_sample=True).shape == (1, 1, 12)
 
     # draws_pd()
-    assert bern_gqs.draws_pd().shape == (1, 10)
+    assert bern_gqs.draws_pd().shape == (1, 13)
     assert (
         bern_gqs.draws_pd(inc_sample=True).shape[1]
         == bern_gqs.previous_fit.optimized_params_pd.shape[1]
@@ -582,9 +628,9 @@ def test_opt_save_iterations(caplog: pytest.LogCaptureFixture):
     assert theta.shape == (1,)
     y_rep = bern_gqs.stan_variable(var='y_rep')
     assert y_rep.shape == (1, 10)
-    theta = bern_gqs.stan_variable(var='theta', inc_warmup=True)
+    theta = bern_gqs.stan_variable(var='theta', inc_iterations=True)
     assert theta.shape == (iters,)
-    y_rep = bern_gqs.stan_variable(var='y_rep', inc_warmup=True)
+    y_rep = bern_gqs.stan_variable(var='y_rep', inc_iterations=True)
     assert y_rep.shape == (iters, 10)
 
 
@@ -615,7 +661,7 @@ def test_opt_request_warmup_none(caplog: pytest.LogCaptureFixture):
         ),
     )
 
-    assert bern_gqs.draws(inc_warmup=True).shape == (1, 1, 10)
+    assert bern_gqs.draws(inc_iterations=True).shape == (1, 1, 10)
 
 
 def test_opt_xarray():
@@ -664,7 +710,7 @@ def test_from_vb():
     assert bern_gqs.draws(inc_sample=True).shape == (1000, 1, 14)
 
     # draws_pd()
-    assert bern_gqs.draws_pd().shape == (1000, 10)
+    assert bern_gqs.draws_pd().shape == (1000, 13)
     assert (
         bern_gqs.draws_pd(inc_sample=True).shape[1]
         == bern_gqs.previous_fit.variational_sample_pd.shape[1]
@@ -672,10 +718,9 @@ def test_from_vb():
     )
 
     # stan_variable
-    theta = bern_gqs.stan_variable(var='theta')
-    # VB behavior is weird: it draws from mean by default?
-    assert theta.shape == (1,)
-    y_rep = bern_gqs.stan_variable(var='y_rep')
+    theta = bern_gqs.stan_variable(var='theta', mean=False)
+    assert theta.shape == (1000,)
+    y_rep = bern_gqs.stan_variable(var='y_rep', mean=False)
     assert y_rep.shape == (1000, 10)
 
 
@@ -725,3 +770,34 @@ def test_vb_xarray():
     bern_gqs = model.generate_quantities(data=jdata, previous_fit=bern_fit)
     with pytest.raises(RuntimeError, match="via Sampling"):
         _ = bern_gqs.draws_xr()
+
+
+@patch(
+    'cmdstanpy.utils.cmdstan.cmdstan_version',
+    MagicMock(return_value=(2, 27)),
+)
+def test_from_non_hmc_old():
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+    bern_model = CmdStanModel(stan_file=stan)
+    jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+    bern_fit_v = bern_model.variational(
+        data=jdata,
+        show_console=True,
+        require_converged=False,
+        seed=12345,
+    )
+
+    # gq_model
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli_ppc.stan')
+    model = CmdStanModel(stan_file=stan)
+
+    with pytest.raises(RuntimeError, match="2.31"):
+        model.generate_quantities(data=jdata, previous_fit=bern_fit_v)
+
+    bern_fit_opt = bern_model.optimize(
+        data=jdata,
+        seed=12345,
+    )
+
+    with pytest.raises(RuntimeError, match="2.31"):
+        model.generate_quantities(data=jdata, previous_fit=bern_fit_opt)
