@@ -89,7 +89,7 @@ def test_bernoulli_good(stanfile: str):
     assert bern_fit.draws().shape == (100, 2, len(BERNOULLI_COLS))
     assert bern_fit.metric_type == 'diag_e'
     assert bern_fit.step_size.shape == (2,)
-    assert bern_fit.metric.shape == (2, 1)
+    assert bern_fit.inv_metric.shape == (2, 1)
 
     assert bern_fit.draws(concat_chains=True).shape == (
         200,
@@ -125,7 +125,7 @@ def test_bernoulli_good(stanfile: str):
     assert bern_sample.shape == (100, 2, len(BERNOULLI_COLS))
     assert bern_fit.metric_type == 'dense_e'
     assert bern_fit.step_size.shape == (2,)
-    assert bern_fit.metric.shape == (2, 1, 1)
+    assert bern_fit.inv_metric.shape == (2, 1, 1)
 
     bern_fit = bern_model.sample(
         data=jdata,
@@ -186,9 +186,7 @@ def test_bernoulli_good(stanfile: str):
 
 
 @pytest.mark.parametrize("stanfile", ["bernoulli.stan"])
-def test_bernoulli_unit_e(
-    stanfile: str, caplog: pytest.LogCaptureFixture
-) -> None:
+def test_bernoulli_unit_e(stanfile: str) -> None:
     stan = os.path.join(DATAFILES_PATH, stanfile)
     bern_model = CmdStanModel(stan_file=stan)
 
@@ -204,18 +202,9 @@ def test_bernoulli_unit_e(
         show_progress=False,
     )
     assert bern_fit.metric_type == 'unit_e'
+    assert bern_fit.inv_metric is None
     assert bern_fit.step_size.shape == (2,)
-    with caplog.at_level(logging.INFO):
-        logging.getLogger()
-        assert bern_fit.metric is None
-    check_present(
-        caplog,
-        (
-            'cmdstanpy',
-            'INFO',
-            'Unit diagnonal metric, inverse mass matrix size unknown.',
-        ),
-    )
+
     assert bern_fit.draws().shape == (100, 2, len(BERNOULLI_COLS))
 
 
@@ -534,7 +523,7 @@ def test_fixed_param_good() -> None:
     )
     assert datagen_fit.runset._args.method == Method.SAMPLE
     assert datagen_fit.metric_type is None
-    assert datagen_fit.metric is None
+    assert datagen_fit.inv_metric is None
     assert datagen_fit.step_size is None
     assert datagen_fit.divergences is None
     assert datagen_fit.max_treedepths is None
@@ -637,7 +626,7 @@ def test_fixed_param_good() -> None:
     assert datagen_fit.column_names == tuple(column_names)
     assert datagen_fit.num_draws_sampling == 100
     assert datagen_fit.draws().shape == (100, 1, len(column_names))
-    assert datagen_fit.metric is None
+    assert datagen_fit.inv_metric is None
     assert datagen_fit.metric_type is None
     assert datagen_fit.step_size is None
 
@@ -859,7 +848,7 @@ def test_validate_big_run() -> None:
     assert fit.column_names == tuple(column_names)
     assert fit.metric_type == 'diag_e'
     assert fit.step_size.shape == (2,)
-    assert fit.metric.shape == (2, 2095)
+    assert fit.inv_metric.shape == (2, 2095)
     assert fit.draws().shape == (1000, 2, 2102)
     assert fit.draws_pd(vars=['phi']).shape == (2000, 2095)
     with raises_nested(ValueError, r'Unknown variable: gamma'):
@@ -1002,53 +991,99 @@ def test_from_csv_no_param_hmc() -> None:
     assert no_parameters_sample.draws_pd().shape == (100, 93)
 
 
-def test_custom_metric() -> None:
+@pytest.mark.parametrize('force_one_process_per_chain', [True, False])
+def test_custom_metric(force_one_process_per_chain: bool) -> None:
     stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
     jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
     bern_model = CmdStanModel(stan_file=stan)
     jmetric = os.path.join(DATAFILES_PATH, 'bernoulli.metric.json')
-    # just test that it runs without error
-    bern_model.sample(
-        data=jdata,
-        chains=2,
-        parallel_chains=2,
-        seed=12345,
-        iter_warmup=100,
-        iter_sampling=200,
-        metric=jmetric,
-    )
     jmetric2 = os.path.join(DATAFILES_PATH, 'bernoulli.metric-2.json')
-    bern_model.sample(
-        data=jdata,
-        chains=2,
-        parallel_chains=2,
-        seed=12345,
-        iter_warmup=100,
-        iter_sampling=200,
-        metric=[jmetric, jmetric2],
-    )
     # read json in as dict
     with open(jmetric) as fd:
         metric_dict_1 = json.load(fd)
     with open(jmetric2) as fd:
         metric_dict_2 = json.load(fd)
-    bern_model.sample(
+    # just test that it runs without error
+    fit1 = bern_model.sample(
+        data=jdata,
+        chains=2,
+        parallel_chains=2,
+        seed=12345,
+        iter_warmup=10,
+        iter_sampling=10,
+        inv_metric=jmetric,
+        force_one_process_per_chain=force_one_process_per_chain,
+    )
+    np.testing.assert_allclose(
+        fit1.inv_metric[0], metric_dict_1['inv_metric'], atol=1e-6
+    )
+    np.testing.assert_allclose(
+        fit1.inv_metric[1], metric_dict_1['inv_metric'], atol=1e-6
+    )
+
+    fit2 = bern_model.sample(
+        data=jdata,
+        chains=2,
+        parallel_chains=2,
+        seed=12345,
+        iter_warmup=10,
+        iter_sampling=10,
+        inv_metric=[jmetric, jmetric2],
+        force_one_process_per_chain=force_one_process_per_chain,
+    )
+    np.testing.assert_allclose(
+        fit2.inv_metric[0], metric_dict_1['inv_metric'], atol=1e-6
+    )
+    np.testing.assert_allclose(
+        fit2.inv_metric[1], metric_dict_2['inv_metric'], atol=1e-6
+    )
+
+    fit3 = bern_model.sample(
         data=jdata,
         chains=4,
         parallel_chains=2,
         seed=12345,
-        iter_warmup=100,
-        iter_sampling=200,
-        metric=metric_dict_1,
+        iter_warmup=10,
+        iter_sampling=10,
+        inv_metric=metric_dict_1,
+        force_one_process_per_chain=force_one_process_per_chain,
     )
-    bern_model.sample(
+    for i in range(4):
+        np.testing.assert_allclose(
+            fit3.inv_metric[i], metric_dict_1['inv_metric'], atol=1e-6
+        )
+    fit4 = bern_model.sample(
         data=jdata,
         chains=2,
         seed=12345,
-        iter_warmup=100,
-        iter_sampling=200,
-        metric=[metric_dict_1, metric_dict_2],
+        iter_warmup=10,
+        iter_sampling=10,
+        inv_metric=[metric_dict_1, metric_dict_2],
+        force_one_process_per_chain=force_one_process_per_chain,
     )
+    np.testing.assert_allclose(
+        fit4.inv_metric[0], metric_dict_1['inv_metric'], atol=1e-6
+    )
+    np.testing.assert_allclose(
+        fit4.inv_metric[1], metric_dict_2['inv_metric'], atol=1e-6
+    )
+
+    fit5 = bern_model.sample(
+        data=jdata,
+        chains=2,
+        seed=12345,
+        iter_warmup=10,
+        iter_sampling=10,
+        inv_metric=[np.array(metric_dict_1['inv_metric']), jmetric2],
+        force_one_process_per_chain=force_one_process_per_chain,
+    )
+    np.testing.assert_allclose(
+        fit5.inv_metric[0], metric_dict_1['inv_metric'], atol=1e-6
+    )
+    np.testing.assert_allclose(
+        fit5.inv_metric[1], metric_dict_2['inv_metric'], atol=1e-6
+    )
+
     with pytest.raises(
         ValueError,
         match='Number of metric files must match number of chains,',
@@ -1058,25 +1093,25 @@ def test_custom_metric() -> None:
             chains=4,
             parallel_chains=2,
             seed=12345,
-            iter_warmup=100,
-            iter_sampling=200,
-            metric=[metric_dict_1, metric_dict_2],
+            iter_warmup=10,
+            iter_sampling=10,
+            inv_metric=[metric_dict_1, metric_dict_2],
+            force_one_process_per_chain=force_one_process_per_chain,
         )
     # metric mismatches - (not appropriate for bernoulli)
     with open(os.path.join(DATAFILES_PATH, 'metric_diag.data.json')) as fd:
         metric_dict_1 = json.load(fd)
     with open(os.path.join(DATAFILES_PATH, 'metric_dense.data.json')) as fd:
         metric_dict_2 = json.load(fd)
-    with pytest.raises(
-        ValueError, match='Found inconsistent "inv_metric" entry'
-    ):
+    with pytest.raises(RuntimeError, match='Error during sampling'):
         bern_model.sample(
             data=jdata,
             chains=2,
             seed=12345,
-            iter_warmup=100,
-            iter_sampling=200,
-            metric=[metric_dict_1, metric_dict_2],
+            iter_warmup=10,
+            iter_sampling=10,
+            inv_metric=[metric_dict_1, metric_dict_2],
+            force_one_process_per_chain=force_one_process_per_chain,
         )
     # metric dict, no "inv_metric":
     some_dict = {"foo": [1, 2, 3]}
@@ -1089,7 +1124,8 @@ def test_custom_metric() -> None:
             seed=12345,
             iter_warmup=100,
             iter_sampling=200,
-            metric=some_dict,
+            inv_metric=some_dict,
+            force_one_process_per_chain=force_one_process_per_chain,
         )
 
 
@@ -1714,6 +1750,12 @@ def test_metadata() -> None:
     assert fit.column_names == col_names
     assert fit.metric_type == 'diag_e'
 
+    assert len(fit.time) == 4
+    for i in range(4):
+        assert 'warmup' in fit.time[i].keys()
+        assert 'sampling' in fit.time[i].keys()
+        assert 'total' in fit.time[i].keys()
+
     assert fit.metadata.cmdstan_config['num_samples'] == 100
     assert fit.metadata.cmdstan_config['thin'] == 1
     assert fit.metadata.cmdstan_config['algorithm'] == 'hmc'
@@ -2077,3 +2119,67 @@ def test_serialization(stanfile='bernoulli.stan'):
     assert set(variables1) == set(variables2)
     for key, value1 in variables1.items():
         np.testing.assert_array_equal(value1, variables2[key])
+
+
+def test_mcmc_create_inits():
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+    bern_model = cmdstanpy.CmdStanModel(stan_file=stan)
+    jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+
+    mcmc = bern_model.sample(data=jdata)
+
+    inits = mcmc.create_inits()
+    assert isinstance(inits, list)
+    assert len(inits) == 4
+    assert isinstance(inits[0], dict)
+    assert 'theta' in inits[0]
+
+    inits_10 = mcmc.create_inits(chains=10)
+    assert isinstance(inits_10, list)
+    assert len(inits_10) == 10
+
+    inits_1 = mcmc.create_inits(chains=1)
+    assert isinstance(inits_1, dict)
+    assert 'theta' in inits_1
+    assert len(inits_1) == 1
+
+    seeded = mcmc.create_inits(seed=1234)
+    seeded2 = mcmc.create_inits(seed=1234)
+    assert all(
+        init1['theta'] == init2['theta']
+        for init1, init2 in zip(seeded, seeded2)
+    )
+
+
+def test_mcmc_init_sampling():
+    stan = os.path.join(DATAFILES_PATH, 'logistic.stan')
+    logistic_model = cmdstanpy.CmdStanModel(stan_file=stan)
+    logistic_data = os.path.join(DATAFILES_PATH, 'logistic.data.R')
+
+    initial_mcmc = logistic_model.sample(data=logistic_data)
+    inits = initial_mcmc.create_inits()
+
+    fit = logistic_model.sample(data=logistic_data, inits=inits)
+
+    assert fit.chains == 4
+    assert fit.draws().shape == (1000, 4, 9)
+
+
+def test_sample_dense_mass_matrix():
+    stan = os.path.join(DATAFILES_PATH, 'linear_regression.stan')
+    jdata = os.path.join(DATAFILES_PATH, 'linear_regression.data.json')
+    linear_model = CmdStanModel(stan_file=stan)
+
+    fit = linear_model.sample(data=jdata, metric="dense_e", chains=2)
+    assert fit.inv_metric is not None
+    assert fit.inv_metric.shape == (2, 3, 3)
+
+
+def test_no_output_draws():
+    stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+    model = cmdstanpy.CmdStanModel(stan_file=stan)
+    data = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+
+    mcmc = model.sample(data=data, iter_sampling=0, save_warmup=False, chains=2)
+    draws = mcmc.draws()
+    assert np.array_equal(draws, np.empty((0, 2, len(mcmc.column_names))))
